# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import gocept.directory.interfaces
import grok
import netaddr
import os.path
import zope.component


grok.context(gocept.directory.interfaces.IDirectory)


class AccessRing0(grok.Permission):
    grok.name('gocept.directory.AccessRing0')


class Ring0(grok.XMLRPC):

    grok.require(AccessRing0)

    def check_consistency(self):
        """Return a list of errors about consistency within the directory."""
        errors = []
        for check in zope.component.subscribers(
                (self.context,),
                gocept.directory.interfaces.IConsistencyCheck):
            errors.extend(check.errors)
        return errors

    def list_virtual_machines(self):
        result = []
        for node in self.context.nodes:
            if not (gocept.directory.interfaces.IVirtualMachine.
                    providedBy(node)):
                continue
            result.append(self.lookup_node(node.name))
        return result

    def list_nodes(self):
        return list(self.lookup_node(n.name) for n in self.context.nodes)

    def lookup_node(self, lookup_name):
        for node in self.context.nodes:
            if node.name == lookup_name:
                break
        else:
            raise KeyError(lookup_name)
        result = {}
        result['classes'] = list(sorted(node.puppet_classes))
        result['name'] = node.name
        # XXX fqdn is deprecated
        result['fqdn'] = node.name
        p = result['parameters'] = {}

        # Generic parameters
        p['id'] = node.global_id
        p['location'] = node.location.abbreviation
        p['machine'] = node.class_
        p['contacts'] = node.contacts
        p['environment'] = node.environment
        p['resource_group'] = node.resource_group.name

        if node.directory_password is not None:
            p['directory_password'] = node.directory_password
            p['directory_ring'] = node.directory_ring

        for nic in node.nics:
            vlan = nic.vlan.class_.short
            p['mac_%s' % vlan] = nic.mac
            for address in nic.addresses:
                if address.netaddr.version == 4:
                    # XXX replace with address instead of index
                    p['ip4_%s' % vlan] = address.index

        # Virtual machine parameter
        # XXX use adapter
        if node.class_ == 'virtual':
            p['rootdev'] = node.rootdev
            p['memory'] = node.memory
            if node.kvm_host is not None:
                p['kvm_host'] = node.kvm_host.name
            else:
                p['kvm_host'] = 'none'

        return result

    def get_password(self, service):
        return self.context.password_by_service(service).clear_text

    def list_people(self):
        result = {}
        for person in self.context.people:
            p = result.setdefault(person.uid, {})
            p['mail'] = person.mail
            p['uid'] = person.uid
            p['cn'] = person.cn
            if person.pager:
                p['pager'] = person.pager
        return result

    def list_groups(self):
        result = {}
        for group in self.context.groups:
            result[group.name] = [x.uid for x in group.members]
        return result

    def list_locations(self):
        return list(x.abbreviation for x in self.context.locations)

    def lookup_location(self, name):
        # DEPRECATED, use lookup_network instead
        for location in self.context.locations:
            if location == name:
                return location
        raise KeyError(name)

    def lookup_networks(self, location, af=''):
        location = self.context.location_by_name(location)
        networks = {}
        if af:
            af = int(af.replace('ipv', ''))
        for vlan in location.vlans:
            networks[vlan.class_.short] = addrs = []
            for network in vlan.networks:
                if af and network.netaddr.version != af:
                    continue
                addrs.append(network.cidr)
            addrs.sort()
            if not addrs:
                del networks[vlan.class_.short]
        return networks

    def list_networks(self):
        networks = set()
        for location in self.context.locations:
            for vlan in location.vlans:
                for network in vlan.networks:
                    networks.add(network.cidr)
        return list(sorted(networks))

    def lookup_interfaces(self, nodename, af_filter=''):
        node = self.context.node_by_name(nodename)
        interfaces = {}
        if af_filter:
            af_filter = int(af_filter.replace('ipv', ''))
        for nic in node.nics:
            addrs = []
            for addr in nic.addresses:
                if af_filter and addr.netaddr.version != af_filter:
                    continue
                addrs.append(addr.cidr)
            interfaces[nic.vlan.class_.short] = sorted(addrs)
        return interfaces

    def lookup_addresses(self, nodename, vlan_filter='', af_filter=''):
        node = self.context.node_by_name(nodename)
        if af_filter:
            af_filter = int(af_filter.replace('ipv', ''))
        addrs = set()
        for nic in node.nics:
            if vlan_filter and nic.vlan.class_.short != vlan_filter:
                continue
            for addr in nic.addresses:
                if af_filter and addr.netaddr.version != af_filter:
                    continue
                addrs.add(addr.cidr)
        return list(sorted(addrs))
