# Copyright (c) 2008-2009 gocept gmbh & co. kg
# See also LICENSE.txt

import gocept.directory.interfaces
import gocept.directory.form
import grok
import megrok.pagelet
import zope.app.authentication.httpplugins
import zope.app.security.interfaces
import zope.dublincore.interfaces


class ManageDirectory(grok.Permission):
    grok.name('gocept.directory.ManageDirectory')


def setup_authentication(pau):
    pau.credentialsPlugins = ('basic-auth',)
    pau.authenticatorPlugins = ('machine-logins',)
    pau['basic-auth'] = (
        zope.app.authentication.httpplugins.HTTPBasicAuthCredentialsPlugin())
    pau['basic-auth'].realm = 'gocept directory'
    pau['machine-logins'] = gocept.directory.auth.MachineLogin()


class Directory(grok.Application, grok.Container):

    grok.local_utility(
        zope.app.authentication.authentication.PluggableAuthentication,
        provides=zope.app.security.interfaces.IAuthentication,
        setup=setup_authentication)

    zope.interface.implements(gocept.directory.interfaces.IDirectory)

    paltry = None

    @property
    def locations(self):
        for obj in self.values():
            if gocept.directory.interfaces.ILocation.providedBy(obj):
                yield obj

    @property
    def vlan_classes(self):
        for obj in self.values():
            if gocept.directory.interfaces.IVLANClass.providedBy(obj):
                yield obj

    @property
    def nodes(self):
        for rg in self.resource_groups:
            for node in rg.nodes:
                yield node

    @property
    def networks(self):
        for location in self.locations:
            for vlan in location.vlans:
                for network in vlan.networks:
                    yield network

    @property
    def resource_groups(self):
        for obj in self.values():
            if gocept.directory.interfaces.IResourceGroup.providedBy(obj):
                yield obj

    @property
    def groups(self):
        for obj in self.values():
            if gocept.directory.interfaces.IGroup.providedBy(obj):
                yield obj

    @property
    def people(self):
        for obj in self.values():
            if gocept.directory.interfaces.IPerson.providedBy(obj):
                yield obj

    def node_by_name(self, name):
        for node in self.nodes:
            if node.name == name:
                return node
        raise KeyError(name)

    def location_by_name(self, name):
        for location in self.locations:
            if location.abbreviation == name:
                return location
        raise KeyError(name)

    def password_by_service(self, name):
        return self['pw-' + name]

    def passwords(self):
        for obj in self.values():
            if gocept.directory.interfaces.IPassword.providedBy(obj):
                return obj


def locate_directory_from(current):
    directory = None
    while current is not None:
        if gocept.directory.interfaces.IDirectory.providedBy(current):
            return current
        current = current.__parent__
    raise ValueError('cannot locate directory object')


class Index(megrok.pagelet.Pagelet):

    grok.require('gocept.directory.ManageDirectory')


class Edit(gocept.directory.form.BasicEditForm):
    pass


class Layout(megrok.pagelet.Layout):
    grok.context(zope.interface.Interface)

    megrok.pagelet.template('layout.pt')


class Import(grok.View):

    grok.require('gocept.directory.ManageDirectory')

    def update(self):
        import_directory(self.context)

    def render(self):
        return 'done'


def import_directory(directory):
    paltry = zope.component.getUtility(
        gocept.directory.interfaces.IPaltryDirectory)

    # Create locations
    for location in paltry.list_locations():
        l = gocept.directory.location.Location()
        l.title = location
        l.abbreviation = location
        directory[location] = l

        # Create networks
        networks = paltry.lookup_networks(location)
        for vlan in l.vlans:
            for network in networks.get(vlan.class_.short, []):
                n = gocept.directory.network.Network()
                n.cidr = network
                vlan[n.cidr.replace('/', '-')] = n

    # Create machines
    nodes = zope.component.getUtility(gocept.directory.interfaces.INodes)

    kvm_assignments = []

    for node in nodes.values():
        location = directory[node['parameters']['location']]
        resource_group = directory[
            'rg-' + node['parameters'].get('resource_group', 'services')]

        if node['parameters']['machine'] == 'virtual':
            f = zope.component.getUtility(zope.component.interfaces.IFactory,
                                          'gocept.directory.VirtualMachine')
            n = f(node['name'], node['parameters']['id'], location,
                  resource_group)
            n.memory = int(node['parameters']['memory'])
            kvm_assignments.append((n, node['parameters']['kvm_host']))
        else:
            f = zope.component.getUtility(zope.component.interfaces.IFactory,
                                          'gocept.directory.PhysicalMachine')
            n = f(node['name'], node['parameters']['id'],
                             location, resource_group)

        n.puppet_classes = set(node['classes'])
        n.environment = node['parameters']['environment']
        n.contacts = node['parameters']['contacts']
        n.directory_password = node['parameters'].get('directory_password')
        n.directory_ring = node['parameters'].get('directory_ring')

        for vlan in location.vlans:
            mac = node['parameters'].get('mac_%s' % vlan.class_.short)
            if mac is None:
                continue
            if vlan.class_.short not in n:
                nic = gocept.directory.nic.NIC()
                nic.vlan = vlan
                n[vlan.class_.short] = nic
            nic = n[vlan.class_.short]
            nic.mac = mac
            for network in vlan.networks:
                if network.netaddr.version != 4:
                    continue
                # We just pick the first IPv4 network we can find
                index = node['parameters'].get('ip4_%s' % vlan.class_.short)
                if index is None:
                    continue
                addr = str(network.netaddr[index])
                addr = gocept.directory.network.Address(addr)
                network.allocate(nic, addr)

    for node, host in kvm_assignments:
        # We delayed those assignments to make sure that all hosts have
        # been created.
        if  host == 'none':
            continue
        node.kvm_host = directory.node_by_name(host)

    # Import persons
    for person in paltry.list_people().values():
        p = gocept.directory.person.Person()
        p.cn = person['cn']
        p.uid = person['uid']
        p.mail = person['mail']
        p.pager = person.get('pager')
        directory[p.uid] = p

    # Import groups
    for group, members in paltry.list_groups().items():
        g = gocept.directory.group.Group()
        g.name = group
        g.members = set()
        for member in members:
            try:
                member = directory[member]
            except KeyError:
                continue
            g.members.add(member)
        directory[group] = g

    # Import passwords
    for name, pw in paltry.list_passwords().items():
        p = gocept.directory.password.Password()
        p.clear_text = pw
        directory['pw-' + name] = p
